 # Modal dialogs with vibrations
 #       (shaking windows)
 #
 # This module provide just a simple procedure
 #  setModalDialog w
 # 
 # Let $w be a toplevel window; by calling
 #  setModalDialog $w
 # you can transform $w in a "shaking" modal dialog-box.
 # - See demo at the end of file -
 #
 # This is a simple experiment for creating modal dialog windows
 #  having a closer native look&feel .
 # In particular, when user clicks outside of the modal-dialog
 #  (another window of the same application)
 #  a bell rings and the modal-dialog vibrates shortly.
 #
 # 2005 A.Buratti
  

 # Tested on:
 #  Windows XP  : OK
 #  Linux (kde) : OK but .. modal-dialog has resize buttons
 #                         (same as for tk_messageBox)
 

namespace eval Dialog {
    namespace export  setModalDialog
}

proc Dialog::setModalDialog {w} {
   if {[winfo viewable [winfo toplevel [winfo parent $w]]] } {
 	wm transient $w [winfo toplevel [winfo parent $w]]
   }
   grab $w
   bind $w <Button-1> "Dialog::_shakeWindow $w %X %Y" 
}

    
 # this proc 'shakes the window' $w when the user does mouse-click
 #  outside of the window $w.
proc Dialog::_shakeWindow { w x y {xincr 2} {ms 25} } { 
   set clickedWidget [winfo containing $x $y]
   if { $clickedWidget == "" } {
        # use an existing widget (.)
       set clickedWidget .
   }
   if { "$w" != [winfo toplevel $clickedWidget] } {
       # $w could be under another app'swindow !
       raise $w
       bell 
        # let the $w window vibrate !
       set geom [split [wm geometry $w] "+"]
       set x [lindex $geom 1]
       set y [lindex $geom 2]

       set xx $x
       set yy $y

       for { set step 0 } {$step < 2} { incr step } {
          incr xx -${xincr}
          wm geometry $w +${xx}+${yy}
          update idletasks
          after $ms
       }
       
       for { set step 0 } {$step < 4} { incr step } {
          incr xx ${xincr}
          wm geometry $w +${xx}+${yy}
          update idletasks
          after $ms
       }

       for { set step 0 } {$step < 2} { incr step } {
          incr xx -${xincr}
          wm geometry $w +${xx}+${yy}
          update idletasks
          after $ms
       }
   }
}

 # example -------------------------------------------- 
	
 # ONLY if we are running in stand-alone mode
 
 if {$argv0 == [info script]} {
      # open 2 similar windows with a button;
      #  pressing the button, you will get a new window
     set w .x1
     toplevel $w ; wm title $w "First Window"
     pack [button $w.b -text "Press me" -width 40 \
           -command "toplevel $w.x ; Dialog::setModalDialog $w.x"
          ] -padx 10 -pady 10

     set w .x2
     toplevel $w ; wm title $w "Second Window"
     pack [button $w.b -text "Press me" -width 40 \
           -command "toplevel $w.x ; Dialog::setModalDialog $w.x"
          ] -padx 10 -pady 10
     lower .

 }

